#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <SDL/SDL.h>
#include <SDL/SDL_net.h>

#include "WebSrv.h"
#include "faf.h"

static int FAF_AcceptConn(WebSrv *ws, const char *user, const char *fname);

int FAF_Run(const char *user, const char *fname, Uint16 port)
{
  TCPsocket sock;
  int       running=1;
  Sint32    tick;
  WebSrv   *ws=NULL;

  if((user==NULL)||(fname==NULL)||(port==0))
    return(-1);

  ws=WebSrv_Create();

  if(ws==NULL)
  {
    fprintf(stderr,"Couldn't create webserver\n");
    return(-1);
  }

  sock=WebSrv_Open(ws,port);
  if(sock==NULL)
  {
    fprintf(stderr,"Couldn't open webserver on port %d\n",port);
    running=-1;
  }

  tick=SDL_GetTicks();

  while(running>0)
  {
    int conns;

    if((SDL_GetTicks()-tick)>DEFAULT_TIMEOUT)
    {
      running=-1;
      continue;
    }

    conns=WebSrv_Poll(ws, 100);
    if(conns<0)
    {
      fprintf(stderr,"Connection Error!\n");
      running=-1;
    }
    else if(conns>0)
    {
      int n;
      for(n=0; n<conns; n++)
        if(FAF_AcceptConn(ws,user,fname)>0)
        {
          fprintf(stderr,"Transferred!\n");
          running=0;
        }
    }
  }  

  WebSrv_Destroy(ws);

  return(running);
}

const char *crop_path(const char *fname)
{
  int len;
  if(fname==NULL)
    return(NULL);

  len=strlen(fname);

  for(len=strlen(fname); len>=0; len--)
    if(fname[len]==DIR_SEP)
      return(fname+len+1);

  return(fname);
}

char *FAF_Webname(const char *fname)
{
  if(fname==NULL) return(NULL);

  fname=crop_path(fname);

  return(WebSrv_enhex(fname));
}

int FileMatch(const char *fname, const char *webname)
{
  int match;

  fname=crop_path(fname);
  webname=WebSrv_dehex((char *)webname);
  match=strcmp(fname,webname);
  free((void *)webname);

  return(match==0);
}

static int FAF_AcceptConn(WebSrv *ws, const char *user, const char *fname)
{
  int auth=0;
  int found=0;

  WebSrvClient *c=WebSrv_Accept(ws);
  if(c==NULL)
    return(-1);

  /* Test to see if filenames match */
//  if(strcmp(fname,(c->url.url)+1)==0)
  if(FileMatch(fname,(c->url.url)+1))
    found=1;

  if(c->auth!=NULL)
  {
    if(strcmp(c->auth,user)==0)
      auth=1;
  }

  if(!found)
  {
    WebSrv_SendHTTPResponse(c, 404);
    WebSrv_ClientSendString(c, "Content-Type: text/html\r\n"
	"Server: File-And-Forget 0.2\r\n"
	"Connection: close\r\n"
	"\r\n"
	"<HTML><BODY><H1>404 Not Found</H1></BODY></HTML>");
    WebSrv_CloseClient(ws,c);
    return(0);
  }
  else if(auth)
  {
    char buf[1024];
    int bytes=0;
    FILE *fp=fopen(fname,"rb");
    if(fp==NULL)
    {
      WebSrv_SendHTTPResponse(c, 404);
      WebSrv_ClientSendString(c, "Content-Type: text/html\r\n"
        "Server: File-And-Forget 0.2\r\n"
        "Connection: close\r\n"
        "\r\n"
        "<HTML><BODY><H1>404 Not Found</H1></BODY></HTML>");
      WebSrv_CloseClient(ws,c);
      return(0);
    }
    else
    {
      WebSrv_SendHTTPResponse(c, 200);
      WebSrv_ClientSendString(c,"Server: File-And-Forget 0.2\r\n"
                                "Connection: keep\r\n"
                                "\r\n");

      bytes=fread(buf,1,1024,fp);
      while(bytes>0)
      {
        if(WebSrv_ClientSend(c,buf,bytes)<bytes)
        {
          WebSrv_CloseClient(ws,c);
          fprintf(stderr,"Sending connection broken\n");
          return(-1);
        }
        bytes=fread(buf,1,1024,fp);
      }      

      WebSrv_CloseClient(ws,c);
      fclose(fp);
      return(1);
    }

  }
  else
  {
    WebSrv_SendHTTPResponse(c, 401);
    WebSrv_ClientSendString(c, "Content-Type: text/html\r\n"
	"Server: File-And-Forget 0.2\r\n"
	"WWW-Authenticate: Basic realm=\"Username and Password Required\"\r\n"
	"Connection: close\r\n"
	"\r\n"
	"<HTML><BODY><H1>Unauthorized</H1></BODY></HTML>");
    WebSrv_CloseClient(ws,c);
    return(0);
  }
}

int FAF_Init()
{
  if(SDL_Init(0)<0)
  {
    fprintf(stderr,"Couldn't initialise SDL: %s\n",SDL_GetError());
    return(-1);
  }
  atexit(SDL_Quit);

  if(SDLNet_Init()<0)
  {
    fprintf(stderr,"Couldn't initialize SDL_Net\n");
    return(-1);
  }
  atexit(SDLNet_Quit);

  return(0);
}
